require 'renderCFN/awsObject'

module RenderCFN
  class LoadBalancer < AwsObject
      def initialize( arguments) #name, lbPort, instancePort, idleTimeout = 60)
        @idleTimeout = arguments[:idleTimeout] or 60
        @name = "#{arguments[:name]}ElasticLoadBalancer"
        @awsObject = { 
          @name => {
            'Type' => 'AWS::ElasticLoadBalancing::LoadBalancer', 
            'Properties' => {
              'SecurityGroups' => [],
              'Scheme' => arguments[:scheme] || 'internal',
              'CrossZone' => true,
              'ConnectionSettings' => {
                'IdleTimeout' => @idleTimeout
              },
              'ConnectionDrainingPolicy' => { 'Enabled' => true, 'Timeout' => 60 },
              'Subnets' => arguments[:subnetList], 
              'Tags' => [
                { 'Key' => 'application', 'Value' => arguments[:serviceTitle] },
                { 'Key' => 'StackName', 'Value' => @@stackName },
              ],
              'HealthCheck' => {
                'Target' => "HTTP:#{arguments[:instancePort]}/status-check",
                'Interval' => 6,
                'Timeout' => 5,
                'UnhealthyThreshold' => 2,
                'HealthyThreshold' => 2,
              },
              'Listeners' => [
                {
                  'Protocol' => 'HTTP', 
                  'LoadBalancerPort' => 80, 
                  'InstancePort' => arguments[:instancePort]
                },
                #{
                #  'Protocol' => 'HTTP', 
                #  'LoadBalancerPort' => 9876, 
                #  'InstancePort' => 9876
                #}
              ]
            }
          }
        }
        if arguments[:lbPort] != 80
          @awsObject[@name]['Properties']['Listeners'].push (
              {
                'Protocol' => 'HTTP', 
                'LoadBalancerPort' => arguments[:lbPort], 
                'InstancePort' => arguments[:instancePort]
              }
          )
        end

      end

      def modifyHealthCheck( newHealthCheck)
        @awsObject[@name]['Properties']['HealthCheck']['Target'] = newHealthCheck
      end

      def addSecurityGroup( securityGroup)
        @awsObject[@name]['Properties']['SecurityGroups'].push( {'Ref' => "#{securityGroup}"} )
      end

      def addListener( lbPort, instancePort)
        @awsObject[@name]['Properties']['Listeners'].push (
          {
            'Protocol' => 'HTTP', 
            'LoadBalancerPort' => lbPort, 
            'InstancePort' => instancePort
          }
        )
      end

      def addSslListener( lbPort, instancePort, certificate)
        @awsObject[@name]['Properties']['Listeners'].push (
          {
            'Protocol' => 'HTTPS', 
            'LoadBalancerPort' => lbPort, 
            'InstancePort' => instancePort,
            'SSLCertificateId' => certificate
          }
        )
      end

  end

end
