#!/usr/bin/env ruby

require 'aws-sdk'
require 'renderCFN/stack'

module RenderCFN
  class MainStack < Stack
    def initialize( arguments)
  
      super( arguments)
  
      @stackID = String.new()
      @templateName = 'main.yml'
      @templateURL = "https://s3.amazonaws.com/#{@@bucket}/stacks/#{@@stackName}/#{@templateName}"
  
      @awsObject = { 
        'AWSTemplateFormatVersion' => '2010-09-09',
        'Description' => arguments[:description] ? "#{arguments[:description]} [renderCFN]" : 'Made with renderCFN',
        'Resources' => {} 
      }

    end
  
    def getAWSStackID( )
      cfn = Aws::CloudFormation::Client.new( )
      stackCall = cfn.describe_stacks(stack_name: @@stackName )
      stackCall['stacks'][0]['stack_id']
    end
  
    def createStack()
      uploadTemplate()
      cfn = Aws::CloudFormation::Client.new( )
      cfn.create_stack( stack_name: @@stackName, template_url: @templateURL, capabilities: ["CAPABILITY_IAM"])
      @@stackID = getAWSStackID()
    end
  
    def updateStack()
      uploadTemplate()
      cfn = Aws::CloudFormation::Client.new( )
      cfn.update_stack( stack_name: @@stackID, template_url: @templateURL, capabilities: ["CAPABILITY_IAM"] )
    end
  
    def deleteStack()
      cfn = Aws::CloudFormation::Client.new( )
      cfn.delete_stack( stack_name: @@stackName)
    end

    def validateStack()
      cfn = Aws::CloudFormation::Client.new( )
      cfn.validate_template( template_url: @templateURL)
    end
  
    def execute()
      case @@action
      when 'create'
        createStack()
      when 'delete'
        deleteStack()
      when 'update'
        updateStack()
      when 'validate'
        validateStack()
      when 'print'
        render()
      end
    end
  
  end

end
